<?php
/* WebDirectory RELEASE 2.0
 * Copyright (C) 2005 Benjamin Heckmann
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

	//Include classes
	include("Logging.class.php");
	include("SecurityItem.class.php");

	//Define class with the directory functions
	class WebDirectory {
		//Class variables
		var $str_baseDirectory;
		var $str_actualDirectory;
		var $array_directoryIndex;
		var $obj_securityitem;
		var $obj_logging;
		var $str_commonerror;
		
		//Class constructor
		function WebDirectory ($str_baseDirectory, $str_logFile) {
			//Define base directory
			$this->str_baseDirectory = $this->GetForcedSlash($str_baseDirectory);

			//Create security item object
			$this->obj_securityitem = new SecurityItem();
			//Create logging object
			$this->obj_logging = new Logging($str_logFile);

			//Define working folder
			$this->FindActualDirectory();
			//Check for file uploads and handle them
			$this->ReceiveFileUpload();
			//Check for extra operations on files and handle them
			$this->HandleFileOperations();
			//Check for extra operations on folders and handle them
			$this->HandleFolderOperations();
			//Read the working folder
			$this->ReadDirectory();
		}
		
		//Class method for defining the working folder
		function FindActualDirectory () {
			//Define base directory as working folder, if not set as GET or SESSION value 
			if (!isset($_GET["dir"]) and !isset($_SESSION["actualdirectory"])) {
				//Log initial folder link
				$this->obj_securityitem->LogOutgoingItem(base64_encode($this->str_baseDirectory));
				//Create link to base directory
				$_GET["dir"] = base64_encode($this->str_baseDirectory);
			}
			
			//Define working folder from its GET value
			if (isset($_GET["dir"])) {
				//Store last folder
				$_SESSION["lastdirectory"][] = $_SESSION["actualdirectory"];
				//Reset actual directory to incoming link
				$_SESSION["actualdirectory"] = $this->GetDecodedSubfolderLink($_GET["dir"]);
			}
			
			//Define last working folder as actual working folder, if option is requested
			if ($_GET["op"] == "last") {
				$str_lastdirectory = array_pop($_SESSION["lastdirectory"]);
				
				if ($str_lastdirectory != "") {
					$_SESSION["actualdirectory"] = $str_lastdirectory;
				}
			}
			
			//Take working folder from SESSION value
			$this->str_actualDirectory = $_SESSION["actualdirectory"];
		}
		
		//Class method for reading the working folder content
		function ReadDirectory () {
			$obj_directoryHandle  = opendir($this->str_actualDirectory);
			//Reset global directory index
			$this->array_directoryIndex = array();			
			
			//Read actual directory
			while (false !== ($str_itemname = readdir($obj_directoryHandle))) {
				array_push($this->array_directoryIndex, $str_itemname);
			}

			//Prepare directory index for usage
			$this->FilterDirectory();
			$this->SortDirectory();
		}
		
		//Class method for filtering unwanted folder entries
		function FilterDirectory () {
			foreach ($this->array_directoryIndex as $int_id => $str_item) {
				//Remove unwanted file and folder names, like "." or ".."
				//Note: "." and ".." are removed for SECURITY reason also
				if ($str_item == ".") {
					unset($this->array_directoryIndex[$int_id]);
				}
				
				if ($str_item == "..") {
					unset($this->array_directoryIndex[$int_id]);
				}
			}
		}
		
		//Class method for sorting the working folder content
		function SortDirectory () {
			sort($this->array_directoryIndex, SORT_STRING);
		}
		
		//Class method for sending the content of a file to the webclient
		function SendFileDownload () {
			//Check for download flag
			if ($_GET["op"] == "download") {
			    //Set file for download
			    $str_file = $this->GetDecodedFileLink($_GET["file"]);
			    
			    //Sent headers for direct download
			    header("Content-Disposition: attachment; filename=".basename($str_file));
			    header("Content-Description: WebDirectory Download");
			    header("Content-Transfer-Encoding: binary");
			
				//Log file download
				$this->obj_logging->Log("Downloaded file; ".$str_file."; ".number_format(filesize($str_file))."; Byte");
				//Send file data
			    readfile($str_file);

 				return true;
			}
			
			return false;
		}
				
		//Class method for handle a file upload
		function ReceiveFileUpload () {
			//Check for succesful file upload
			if (isset($_POST["upload"]) and $_FILES["uploadfile"]["error"] == UPLOAD_ERR_OK) {
				//Log file upload
				$this->obj_logging->Log("Uploaded file; ".$this->str_actualDirectory.$_FILES["uploadfile"]["name"]);
				//Move new file to actual directory 
				move_uploaded_file($_FILES["uploadfile"]["tmp_name"], $this->str_actualDirectory.$_FILES["uploadfile"]["name"]);
			}
			
			if (isset($_POST["upload"]) and $_FILES["uploadfile"]["error"] != UPLOAD_ERR_OK) {
				//Report error to user
				$this->str_commonerror = "File upload failed.";
				
				switch ($_FILES["uploadfile"]["error"]) {
				   case UPLOAD_ERR_INI_SIZE:
				       $this->str_commonerror .= " Maximum upload size exceeded.";
					   break;
				   case UPLOAD_ERR_PARTIAL:
				       $this->str_commonerror .= " File was only uploaded partial.";
					   break;
				   case UPLOAD_ERR_NO_FILE:
				       $this->str_commonerror .= " No file was send.";
					   break;
				   case UPLOAD_ERR_NO_TMP_DIR:
				       $this->str_commonerror .= " SERVER SIDE ERROR: Access to temporary folder failed.";
					   break;
				}
			}
 		}
 		
		//Class method for creating a valid link to a subfolder of the working folder as html code
		function GetEncodedSubfolderLink ($str_folder, $bol_self = true) {
			//Log outgoing folder link
			$this->obj_securityitem->LogOutgoingItem(base64_encode($this->GetSubfolderLink($str_folder)));
			//Create link to script as html code
			if ($bol_self) $str_self = $_SERVER["PHP_SELF"]."?dir=";
			//Create folder link as html code
			return $str_self.base64_encode($this->GetSubfolderLink($str_folder));
		}
		
		//Class method for recreating a link to a subfolder from html code
		function GetDecodedSubfolderLink ($str_link) {
			//Check directory link
			$this->obj_securityitem->CheckIncomingItem($str_link);
			//Decode directory link
			return base64_decode($str_link);
		}
		
		//Class method for creating a valid link to a subfolder of the working folder
		function GetSubfolderLink ($str_folder) {
			return $this->GetForcedSlash($this->str_actualDirectory.$str_folder);
		}
		
		//Class method for removing original root directory path from actual directory
		function GetSubfolderPath ($str_subfolder) {
			return substr($str_subfolder, strlen($this->str_baseDirectory));
		}
		
		//Class method for ensuring a path always ends with a slash
		function GetForcedSlash ($str_path) {
			//Force directorypaths ends with "/"
			if (substr($str_path, -1) != "/") {
				$str_path .= "/";
			}
			
			return $str_path;
		}
		
		//Class method for working folder file only list as html code
		function GetOutputFiles () {
			//Walk through directory index
			foreach ($this->array_directoryIndex as $str_item) {
				//Check for files
				if (is_file($this->str_actualDirectory.$str_item)) { 
					$str_output .= ("<p><a href='".$this->GetEncodedFileLink($str_item)."&amp;op=download'>".$str_item."</a>&nbsp;&nbsp;&nbsp;".$this->GetFileInformations($str_item)."&nbsp;&nbsp;&nbsp;".$this->GetFileOperations($str_item)."</p>");
				}
			}
			
			//Add headline
			if ($str_output != "") {
				$str_output = "<h3>File List</h3>".$str_output;
			}
			
			return $str_output;
		}
		
		//Class method for creating a valid link to a file of the working folder as html code
		function GetEncodedFileLink ($str_file) {
			//Log outgoing file link
			$this->obj_securityitem->LogOutgoingItem(base64_encode($this->GetFileLink($str_file)));
			//Create file link as html code
			return $_SERVER["PHP_SELF"]."?file=".base64_encode($this->GetFileLink($str_file));
		}
		
		//Class method for recreating a link from html code
		function GetDecodedFileLink ($str_link) {
		    //Check incoming file link
			$this->obj_securityitem->CheckIncomingItem($str_link);
		    //Set file
			return base64_decode($str_link);
		}
		
		//Class method for creating a valid link to a file of the working folder
		function GetFileLink ($str_file) {
			return $this->GetForcedSlash($this->str_actualDirectory).$str_file;
		}
		
		//Class method for creating additional file information as text
		function GetFileInformations ($str_file) {
			return number_format(filesize($this->str_actualDirectory.$str_file), 0, ',', '.')." Byte";
		}
		
		//Class method for creating html code for additional file operations
		function GetFileOperations ($str_file) {
			//Add file deletion operation
			$str_delete = "<a href='".$this->GetEncodedFileLink($str_file)."&amp;op=predelete'>Delete</a>";

			//Force secondary deletion query
			if ($_SESSION["predelete"] == $this->GetFileLink($str_file)) {
				$str_delete = "DELETE FILE? <a href='".$this->GetEncodedFileLink($str_file)."&amp;op=delete'>Yes</a> <a href='".$this->GetEncodedFileLink($str_file)."&amp;op=revdelete'>No</a>";
			}
			
			//Additional file operations may be added here ...
			
			return $str_delete;
		}
		
		//Class method for executing the additional file operations
		function HandleFileOperations () {
			//Register first deletion query
			if ($_GET["op"] == "predelete") {
				//Store filename for later identification of the secondary deletion query
				$_SESSION["predelete"] = $this->GetDecodedFileLink($_GET["file"]);
			}
			
			//Check secondary deletion query is true
			if ($_GET["op"] == "delete" and isset($_SESSION["predelete"])) {
				//reset first deletion query register
				unset($_SESSION["predelete"]);
			    
			    //Get filename
			    $str_file = $this->GetDecodedFileLink($_GET["file"]);

				//Delete file
				if (@unlink($str_file)) {
					//Log file deletion
					$this->obj_logging->Log("Deleted file; ".$str_file);
				}
				else {
					//Report error to user
					$this->str_commonerror = "File could not be deleted.";
				}
			}

			//Reset to first deletion query
			if ($_GET["op"] == "revdelete" and isset($_SESSION["predelete"])) {
				//reset first deletion query register
				unset($_SESSION["predelete"]);
			}

			//Additional file operation handling may be added here ...
		}
		
		//Class method for folder list as html selectbox options
		function GetFolderList () {
			//Walk through directory index
			foreach ($this->array_directoryIndex as $str_item) {
				//Check for folders
				if (is_dir($this->str_actualDirectory.$str_item)) { 
					$str_subfolders .= ("<option value='".$this->GetEncodedSubfolderLink($str_item, false)."'>".$str_item."</option>");
				}
			}

			return $str_subfolders;
		}

		//Class method for folder editor
		function GetFolderOperations () {
			//Handle editor activation link
			if ($_GET["diredit"] == "inactive" or !$_SESSION["diredit"]) {
				//Create editor activation link
				$str_editor = "<p class='left'><a href='".$_SERVER["PHP_SELF"]."?diredit=active'>Folder Editor &gt;&gt;</a></p>";
			}
			
			//Handle editor inactivation link
			if ($_GET["diredit"] == "active" or $_SESSION["diredit"]) {
				//Create editor inactivation link
				$str_editor = "<p class='left'><a href='".$_SERVER["PHP_SELF"]."?diredit=inactive'>Folder Editor &lt;&lt;</a></p>";
			}

			//Create editor
			if (($_SESSION["diredit"])) {
				$str_editor .= "<h3 class='left'>Folder Editor</h3>";
				//Create folder creation interface
				$str_editor .= "<form method='post' action='".$_SERVER["PHP_SELF"]."'><p class='left'>New Subfolder <input type='text' name='adddirname' size='20'> <input type='submit' value='Add' name='adddir'></p></form>";
				
				//Create folder deletion interface
				$str_delete = "<form method='post' action='".$_SERVER["PHP_SELF"]."'><p class='left'>Remove Subfolder <select size='1' name='deldirname'>".$this->GetFolderList()."</select> <input type='submit' value='Delete' name='predeldir'></p></form>";
				
				//Add security request before deletion
				if ($_SESSION["dirdel"] != "") {
					$str_delete = "<p class='left'>DELETE FOLDER ".$this->GetSubfolderPath($_SESSION["dirdel"])."? <a href='".$_SERVER["PHP_SELF"]."?opdir=deldir'>Yes</a> <a href='".$_SERVER["PHP_SELF"]."?opdir=revdeldir'>No</a></p>";	
				} 
				
				$str_editor .= $str_delete;
			}
			
			//Additional folder operations may be added here ...

			return $str_editor;
		}
		
		//Class method for executing the additional folder operations
		function HandleFolderOperations () {
			//Handle editor activation link
			if ($_GET["diredit"] == "inactive" or !isset($_SESSION["diredit"])) {
				//Register editor inactivation
				unset($_SESSION["diredit"]);
			}
			
			//Handle editor deactivation link
			if ($_GET["diredit"] == "active" or isset($_SESSION["diredit"])) {
				//Register editor activation
				$_SESSION["diredit"] = true;
			}

			//Handle folder predeletion activation
			if (isset($_POST["predeldir"])) {
				//Register folder deletion request
				$_SESSION["dirdel"] = $this->GetDecodedSubfolderLink($_POST["deldirname"]);
			}

			//Handle folder predeletion deactivation
			if ($_GET["opdir"] == "revdeldir" and isset($_SESSION["dirdel"])) {
				//Unregister folder deletion request
				unset($_SESSION["dirdel"]);
			}

			//Handle folder deletion
			if ($_GET["opdir"] == "deldir" and isset($_SESSION["dirdel"])) {
				//Delete folder
				if (@rmdir($_SESSION["dirdel"])) {
					//Log folder deletion
					$this->obj_logging->Log("Deleted folder; ".$_SESSION["dirdel"]);
				}
				else {
					//Report error to user
					$this->str_commonerror = "Subfolder could not be removed. Possibly the folder ist not empty?";
				}

				//Unregister folder deletion request
				unset($_SESSION["dirdel"]);
			}

			//Handle folder creation
			if (isset($_POST["adddir"])) {
				//Create folder
				if (@mkdir($this->GetSubfolderLink($_POST["adddirname"]))) {
					//Log folder creation
					$this->obj_logging->Log("Created folder; ".$this->GetSubfolderLink($_POST["adddirname"]));
				}
				else {
					//Report error to user
					$this->str_commonerror = "Subfolder could not be created.";
				}
			}

			//Additional folder operation handling may be added here ...
		}
		
		//Class method for back-link option as html code
		function GetOutputFindLastDirectory () {
			return "<h2><a href='".$_SERVER["PHP_SELF"]."?op=last'>BACK</a></h2>";
		}
		
		//Class method for directory path to working folder as html code
		function GetOutputActualDirectory () {
			//Create visual directory path
			return "<h1>Root Directory/".$this->GetSubfolderPath($this->str_actualDirectory)."</h1>";
		}
		
 		function GetOutputPhpEnvironment () {
 			//Determine maximum upload size
 			$str_output = "Maximum upload size: ".ini_get("upload_max_filesize")." Byte";
 			
 			//Determin maximum upload time
 			$str_output .= " - Maximum upload/download time:";
 			if (ini_get("max_input_time") != ini_get("max_execution_time")) $str_output .= " ".(ini_get("max_input_time")/60)." / ".(ini_get("max_execution_time")/60)." Minutes"; 
 			else $str_output .= " ".(ini_get("max_execution_time")/60)." Minutes"; 
 			
 			//Determine free diskspace
 			$str_output .= " - Free Diskspace: ".number_format((disk_free_space($this->str_baseDirectory)/1024), 0, ',', '.')." MB";
 			
 			//Return gathered information about the environment
 			return "<p class='left'>".$str_output."</p>";
 		}

		//Class method for creating file upload support as html code
		function GetOutputFileUpload () {
			return "<form method='post' action='".$_SERVER["PHP_SELF"]."' enctype='multipart/form-data'><h3 class='left'>New File <input type='file' name='uploadfile' size='20'> <input type='submit' value='Add' name='upload'></h3></form>";
		}
		
		//Class method for creating a folder editor as html code
		function GetOutputFolderEditor () {
			return $this->GetFolderOperations();
		}
		
		//Class method for working folder folder only list as html code
		function GetOutputFolders () {
			//Walk through directory index
			foreach ($this->array_directoryIndex as $str_item) {
				//Check for folders
				if (is_dir($this->str_actualDirectory.$str_item)) { 
					$str_subfolders .= ("<h2><a href='".$this->GetEncodedSubfolderLink($str_item)."'>".$str_item."</a></h2>");
				}
			}
			
			//Add headline
			if ($str_subfolders != "") {
				$str_output .= "<h3>Subfolder List</h3>".$str_subfolders;
			}
			
			return $str_output;
		}

		//Class method for error output as html code
		function GetOutputError () {
			return "<p class='error'>".$this->str_commonerror."</p>";
		}
	}
?>